package com.njyunzhi.pet_identity.common;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.builder.ExcelWriterBuilder;
import com.alibaba.fastjson.JSON;
import lombok.Data;
import net.lingala.zip4j.ZipFile;
import net.lingala.zip4j.exception.ZipException;
import net.lingala.zip4j.model.ZipParameters;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.nio.channels.Channels;
import java.nio.channels.ReadableByteChannel;
import java.nio.channels.WritableByteChannel;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.List;

@Component
public class ZipUtil {

    @Autowired
    HttpUtils httpUtils;

    private String today() {
        String formatterStr = "yyyy-MM-dd";
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern(formatterStr);
        return LocalDateTime.now(ZoneId.of("Asia/Shanghai")).format(formatter);
    }

    public ZipFile zipLink(List<RFile> files) throws IOException {
        ZipFile zipFile = new ZipFile(today() + ".zip");

        if (files == null || files.size() < 1) {
            return zipFile;
        }

        for (RFile f : files) {
            ZipParameters zipParameters = new ZipParameters();
            zipParameters.setFileNameInZip(f.getName());
            InputStream inputStream = httpUtils.download(f.getUrl());
            zipFile.addStream(inputStream, zipParameters);
            inputStream.close();
        }

        return zipFile;
    }

    private long copyStream(InputStream input, OutputStream output) throws IOException {
        ReadableByteChannel inputChannel = Channels.newChannel(input);
        WritableByteChannel outputChannel = Channels.newChannel(output);
        ByteBuffer buffer = ByteBuffer.allocateDirect(10240);
        long size = 0;

        while (inputChannel.read(buffer) != -1) {
            buffer.flip();
            size += outputChannel.write(buffer);
            buffer.clear();
        }

        return size;
    }

    public void flush(HttpServletResponse response, ZipFile zipFile) throws IOException {
        try {
            String fileName = zipFile.getFile().getName();
            InputStream inputStream = new FileInputStream(zipFile.getFile());

            response.setContentType("application/zip");
            response.setCharacterEncoding("utf-8");
            response.setHeader("Content-disposition", "attachment;filename="+StringUtils.urlEncode(fileName));

            response.setContentLength(zipFile.getBufferSize());
            copyStream(inputStream, response.getOutputStream());
        } catch (Exception e) {
            e.printStackTrace();

            // 重置response
            response.reset();
            response.setContentType("application/json");
            response.setCharacterEncoding("utf-8");
            ResponseBean res = ResponseBean.error(e.getMessage(), ResponseBean.ERROR_UNAVAILABLE);
            response.getWriter().println(JSON.toJSONString(res));
        }
    }

    @Data
    public static class RFile {
        String url;
        String name;
    }
}
